@@ -26,7 +26,7 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id: boot.h,v 1.27.2.1 2008/03/16 15:50:03 arcanum Exp $ */
+/* $Id: boot.h,v 1.27.2.5 2009/04/25 21:29:10 arcanum Exp $ */
 
 #ifndef _AVR_BOOT_H_
 #define _AVR_BOOT_H_    1
@@ -115,6 +115,16 @@
 #  error AVR processor does not provide bootloader support!
 #endif
 
+
+/* Check for SPM Enable bit. */
+#if defined(SPMEN)
+#  define __SPM_ENABLE  SPMEN
+#elif defined(SELFPRGEN)
+#  define __SPM_ENABLE  SELFPRGEN
+#else
+#  error Cannot find SPM Enable bit definition!
+#endif
+
 /** \ingroup avr_boot
     \def BOOTLOADER_SECTION
 
@@ -173,7 +183,7 @@
     \def boot_spm_busy()
     Check if the SPM instruction is busy. */
 
-#define boot_spm_busy()               (__SPM_REG & (uint8_t)_BV(SPMEN))
+#define boot_spm_busy()               (__SPM_REG & (uint8_t)_BV(__SPM_ENABLE))
 
 /** \ingroup avr_boot
     \def boot_spm_busy_wait()
@@ -181,27 +191,30 @@
 
 #define boot_spm_busy_wait()          do{}while(boot_spm_busy())
 
-#define __BOOT_PAGE_ERASE         (_BV(SPMEN) | _BV(PGERS))
-#define __BOOT_PAGE_WRITE         (_BV(SPMEN) | _BV(PGWRT))
-#define __BOOT_PAGE_FILL          _BV(SPMEN)
-#define __BOOT_RWW_ENABLE         (_BV(SPMEN) | _BV(__COMMON_ASRE))
-#define __BOOT_LOCK_BITS_SET      (_BV(SPMEN) | _BV(BLBSET))
+#define __BOOT_PAGE_ERASE         (_BV(__SPM_ENABLE) | _BV(PGERS))
+#define __BOOT_PAGE_WRITE         (_BV(__SPM_ENABLE) | _BV(PGWRT))
+#define __BOOT_PAGE_FILL          _BV(__SPM_ENABLE)
+#define __BOOT_RWW_ENABLE         (_BV(__SPM_ENABLE) | _BV(__COMMON_ASRE))
+#if defined(BLBSET)
+#define __BOOT_LOCK_BITS_SET      (_BV(__SPM_ENABLE) | _BV(BLBSET))
+#elif defined(RFLB)  /* Some devices have RFLB defined instead of BLBSET. */
+#define __BOOT_LOCK_BITS_SET      (_BV(__SPM_ENABLE) | _BV(RFLB))
+#endif
 
 #define __boot_page_fill_normal(address, data)   \
 (__extension__({                                 \
     __asm__ __volatile__                         \
     (                                            \
         "movw  r0, %3\n\t"                       \
-        "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         "clr  r1\n\t"                            \
         :                                        \
         : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
           "r" ((uint8_t)__BOOT_PAGE_FILL),       \
-          "r" ((uint16_t)address),               \
+          "z" ((uint16_t)address),               \
           "r" ((uint16_t)data)                   \
-        : "r0", "r30", "r31"                     \
+        : "r0"                                   \
     );                                           \
 }))
 
@@ -210,7 +223,6 @@
     __asm__ __volatile__                         \
     (                                            \
         "movw  r0, %3\n\t"                       \
-        "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
@@ -219,9 +231,9 @@
         :                                        \
         : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
           "r" ((uint8_t)__BOOT_PAGE_FILL),       \
-          "r" ((uint16_t)address),               \
+          "z" ((uint16_t)address),               \
           "r" ((uint16_t)data)                   \
-        : "r0", "r30", "r31"                     \
+        : "r0"                                   \
     );                                           \
 }))
 
@@ -249,14 +261,12 @@
 (__extension__({                                 \
     __asm__ __volatile__                         \
     (                                            \
-        "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         :                                        \
         : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
           "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
-          "r" ((uint16_t)address)                \
-        : "r30", "r31"                           \
+          "z" ((uint16_t)address)                \
     );                                           \
 }))
 
@@ -264,7 +274,6 @@
 (__extension__({                                 \
     __asm__ __volatile__                         \
     (                                            \
-        "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
@@ -272,8 +281,7 @@
         :                                        \
         : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
           "r" ((uint8_t)__BOOT_PAGE_ERASE),      \
-          "r" ((uint16_t)address)                \
-        : "r30", "r31"                           \
+          "z" ((uint16_t)address)                \
     );                                           \
 }))
 
@@ -298,14 +306,12 @@
 (__extension__({                                 \
     __asm__ __volatile__                         \
     (                                            \
-        "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         :                                        \
         : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
           "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
-          "r" ((uint16_t)address)                \
-        : "r30", "r31"                           \
+          "z" ((uint16_t)address)                \
     );                                           \
 }))
 
@@ -313,7 +319,6 @@
 (__extension__({                                 \
     __asm__ __volatile__                         \
     (                                            \
-        "movw r30, %2\n\t"                       \
         "sts %0, %1\n\t"                         \
         "spm\n\t"                                \
         ".word 0xffff\n\t"                       \
@@ -321,8 +326,7 @@
         :                                        \
         : "i" (_SFR_MEM_ADDR(__SPM_REG)),        \
           "r" ((uint8_t)__BOOT_PAGE_WRITE),      \
-          "r" ((uint16_t)address)                \
-        : "r30", "r31"                           \
+          "z" ((uint16_t)address)                \
     );                                           \
 }))
 
@@ -379,12 +383,12 @@
 
      If bits 5..2 in R0 are cleared (zero), the corresponding Boot Lock bit
      will be programmed if an SPM instruction is executed within four cycles
-     after BLBSET and SPMEN are set in SPMCR. The Z-pointer is don t care
-     during this operation, but for future compatibility it is recommended to
-     load the Z-pointer with $0001 (same as used for reading the Lock
-     bits). For future compatibility It is also recommended to set bits 7, 6,
-     1, and 0 in R0 to 1 when writing the Lock bits. When programming the Lock
-     bits the entire Flash can be read during the operation. */
+     after BLBSET and SPMEN (or SELFPRGEN) are set in SPMCR. The Z-pointer is 
+     don't care during this operation, but for future compatibility it is 
+     recommended to load the Z-pointer with $0001 (same as used for reading the 
+     Lock bits). For future compatibility It is also recommended to set bits 7, 
+     6, 1, and 0 in R0 to 1 when writing the Lock bits. When programming the 
+     Lock bits the entire Flash can be read during the operation. */
 
 #define __boot_lock_bits_set(lock_bits)                    \
 (__extension__({                                           \
@@ -427,9 +431,9 @@
 /*
    Reading lock and fuse bits:
 
-     Similarly to writing the lock bits above, set BLBSET and SPMEN bits
-     in __SPMREG, and then (within four clock cycles) issue an LPM
-     instruction.
+     Similarly to writing the lock bits above, set BLBSET and SPMEN (or 
+     SELFPRGEN) bits in __SPMREG, and then (within four clock cycles) issue an 
+     LPM instruction.
 
      Z address:       contents:
      0x0000           low fuse bits
@@ -481,15 +485,12 @@
     uint8_t __result;                                      \
     __asm__ __volatile__                                   \
     (                                                      \
-        "ldi r30, %3\n\t"                                  \
-        "ldi r31, 0\n\t"                                   \
         "sts %1, %2\n\t"                                   \
         "lpm %0, Z\n\t"                                    \
         : "=r" (__result)                                  \
         : "i" (_SFR_MEM_ADDR(__SPM_REG)),                  \
           "r" ((uint8_t)__BOOT_LOCK_BITS_SET),             \
-          "M" (address)                                    \
-        : "r0", "r30", "r31"                               \
+          "z" ((uint16_t)address)                          \
     );                                                     \
     __result;                                              \
 }))
@@ -505,7 +506,7 @@
     \note The values are MCU type dependent.
 */
 
-#define __BOOT_SIGROW_READ (_BV(SPMEN) | _BV(SIGRD))
+#define __BOOT_SIGROW_READ (_BV(__SPM_ENABLE) | _BV(SIGRD))
 
 #define boot_signature_byte_get(addr) \
 (__extension__({		      \
