@@ -28,7 +28,7 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE. */
 
-/* $Id: io8535.h,v 1.9 2005/10/30 22:11:23 joerg_wunsch Exp $ */
+/* $Id: io8535.h,v 1.9.4.4 2009/02/11 18:05:23 arcanum Exp $ */
 
 /* avr/io8535.h - definitions for AT90S8535 */
 
@@ -539,9 +539,30 @@
 #define    EERE     0
 
 /* Constants */
-#define    RAMEND   0x25F    /*Last On-Chip SRAM location*/
-#define    XRAMEND  0x25F
-#define    E2END    0x1FF
-#define    FLASHEND 0x1FFF
+#define RAMEND     0x25F    /*Last On-Chip SRAM location*/
+#define XRAMEND    RAMEND
+#define E2END      0x1FF
+#define E2PAGESIZE 0
+#define FLASHEND   0x1FFF
+
+
+/* Fuses */
+#define FUSE_MEMORY_SIZE 1
+
+/* Low Fuse Byte */
+#define FUSE_SPIEN (unsigned char)~_BV(1)  /* Serial Program Downloading Enabled */
+#define FUSE_FSTRT (unsigned char)~_BV(2)  /* Short Start-up time selected */
+#define LFUSE_DEFAULT (0xFF)
+
+
+/* Lock Bits */
+#define __LOCK_BITS_EXIST
+
+
+/* Signature */
+#define SIGNATURE_0 0x1E
+#define SIGNATURE_1 0x93
+#define SIGNATURE_2 0x03
+
 
 #endif /* _AVR_IO8535_H_ */
