@@ -1,4 +1,4 @@
-/* Copyright (c) 2002, 2005, Joerg Wunsch
+/* Copyright (c) 2002, 2005, 2007 Joerg Wunsch
    All rights reserved.
 
    Portions of documentation Copyright (c) 1990, 1991, 1993
@@ -33,7 +33,7 @@
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
 
-  $Id: stdio.h,v 1.24.2.1 2006/03/02 20:08:50 joerg_wunsch Exp $
+  $Id: stdio.h,v 1.29.2.1 2008/02/23 08:59:27 dmix Exp $
 */
 
 #ifndef _STDIO_H_
@@ -48,6 +48,7 @@
 #define __need_size_t
 #include <stddef.h>
 
+/** \file */
 /** \defgroup avr_stdio <stdio.h>: Standard IO facilities
     \code #include <stdio.h> \endcode
 
@@ -139,16 +140,12 @@
     \anchor stdio_without_malloc
     <h3>Running stdio without malloc()</h3>
 
-    By default, fdevopen() as well as the floating-point versions of
-    the printf and scanf family require malloc().  As this is often
+    By default, fdevopen() requires malloc().  As this is often
     not desired in the limited environment of a microcontroller, an
     alternative option is provided to run completely without malloc().
 
     The macro fdev_setup_stream() is provided to prepare a
-    user-supplied FILE buffer for operation with stdio.  If
-    floating-point operation is desired, a user-supplied buffer can as
-    well be passed for the internal buffering for the floating-point
-    numbers (and processing of \%[ scanf data).
+    user-supplied FILE buffer for operation with stdio.
 
     <h4>Example</h4>
 
@@ -497,17 +494,18 @@ extern int	fclose(FILE *__stream);
       
    -   An optional decimal digit string specifying a minimum field width.
        If the converted value has fewer characters than the field width, it
-       will be padded with spaces on the left (or right, if the left-adjust
-       ment flag has been given) to fill out the field width.
+       will be padded with spaces on the left (or right, if the left-adjustment
+       flag has been given) to fill out the field width.
    -   An optional precision, in the form of a period . followed by an
        optional digit string.  If the digit string is omitted, the
        precision is taken as zero.  This gives the minimum number of
        digits to appear for d, i, o, u, x, and X conversions, or the
        maximum number of characters to be printed from a string for \c s
        conversions.
-   -   An optional \c l length modifier, that specifies that the
+   -   An optional \c l or \c h length modifier, that specifies that the
        argument for the d, i, o, u, x, or X conversion is a \c "long int"
-       rather than \c int.
+       rather than \c int. The \c h is ignored, as \c "short int" is
+       equivalent to \c int.
    -   A character that specifies the type of conversion to be applied.
 
    The conversion specifiers and their meanings are:
@@ -571,8 +569,8 @@ extern int	fclose(FILE *__stream);
    selected using linker options.  The default vfprintf() implements
    all the mentioned functionality except floating point conversions.
    A minimized version of vfprintf() is available that only implements
-   the very basic integer and string conversion facilities, but none
-   of the additional options that can be specified using conversion
+   the very basic integer and string conversion facilities, but only
+   the \c # additional option can be specified using conversion
    flags (these flags are parsed correctly from the format
    specification, but then simply ignored).  This version can be
    requested using the following \ref gcc_minusW "compiler options":
@@ -589,9 +587,21 @@ extern int	fclose(FILE *__stream);
    \endcode
 
    \par Limitations:
-   - The specified width and precision can be at most 127.
-   - For floating-point conversions, trailing digits will be lost if
-     a number close to DBL_MAX is converted with a precision > 0.
+   - The specified width and precision can be at most 255.
+
+   \par Notes:
+   - For floating-point conversions, if you link default or minimized
+     version of vfprintf(), the symbol \c ? will be output and double
+     argument will be skiped. So you output below will not be crashed.
+     For default version the width field and the "pad to left" ( symbol
+     minus ) option will work in this case.
+   - The \c hh length modifier is ignored (\c char argument is
+     promouted to \c int). More exactly, this realization does not check
+     the number of \c h symbols.
+   - But the \c ll length modifier will to abort the output, as this
+     realization does not operate \c long \c long arguments.
+   - The variable width or precision field (an asterisk \c * symbol)
+     is not realized and will to abort the output.
 
  */
 
@@ -862,149 +872,6 @@ extern int	ferror(FILE *__stream);
 #define ferror(s) ((s)->flags & __SERR)
 #endif /* !defined(__DOXYGEN__) */
 
-/**
-   Formatted input.  This function is the heart of the \c scanf
-   family of functions.
-
-   Characters are read from \c stream and processed in a way
-   described by \c fmt.  Conversion results will be assigned to the
-   parameters passed via \c ap.
-
-   The format string \c fmt is scanned for conversion specifications.
-   Anything that doesn't comprise a conversion specification is taken
-   as text that is matched literally against the input.  White space
-   in the format string will match any white space in the data
-   (including none), all other characters match only itself.
-   Processing is aborted as soon as the data and format string no
-   longer match, or there is an error or end-of-file condition on
-   \c stream.
-
-   Most conversions skip leading white space before starting the
-   actual conversion.
-
-   Conversions are introduced with the character \b %.  Possible
-   options can follow the \b %:
-
-   - a \c * indicating that the conversion should be performed but
-     the conversion result is to be discarded; no parameters will
-     be processed from \c ap,
-   - the character \c h indicating that the argument is a pointer
-     to <tt>short int</tt> (rather than <tt>int</tt>),
-   - the character \c l indicating that the argument is a pointer
-     to <tt>long int</tt> (rather than <tt>int</tt>, for integer
-     type conversions), or a pointer to \c double (for floating
-     point conversions).
-
-   In addition, a maximal field width may be specified as a nonzero
-   positive decimal integer, which will restrict the conversion to at
-   most this many characters from the input stream.  This field width
-   is limited to at most 127 characters which is also the default
-   value (except for the <tt>%c</tt> conversion that defaults to 1).
-
-   The following conversion flags are supported:
-
-   - \c % Matches a literal \c % character.  This is not a conversion.
-   - \c d Matches an optionally signed decimal integer; the next
-     pointer must be a pointer to \c int.
-   - \c i Matches an optionally signed integer; the next pointer must
-     be a pointer to \c int.  The integer is read in base 16 if it
-     begins with \b 0x or \b 0X, in base 8 if it begins with \b 0, and
-     in base 10 otherwise.  Only characters that correspond to the
-     base are used.
-   - \c o Matches an octal integer; the next pointer must be a pointer to
-     <tt>unsigned int</tt>.
-   - \c u Matches an optionally signed decimal integer; the next
-     pointer must be a pointer to <tt>unsigned int</tt>.
-   - \c x Matches an optionally signed hexadecimal integer; the next
-     pointer must be a pointer to <tt>unsigned int</tt>.
-   - \c f Matches an optionally signed floating-point number; the next
-     pointer must be a pointer to \c float.
-   - <tt>e, g, E, G</tt> Equivalent to \c f.
-   - \c s
-     Matches a sequence of non-white-space characters; the next pointer
-     must be a pointer to \c char, and the array must be large enough to
-     accept all the sequence and the terminating \c NUL character.  The
-     input string stops at white space or at the maximum field width,
-     whichever occurs first.
-   - \c c
-     Matches a sequence of width count characters (default 1); the next
-     pointer must be a pointer to \c char, and there must be enough room
-     for all the characters (no terminating \c NUL is added).  The usual
-     skip of leading white space is suppressed.  To skip white space
-     first, use an explicit space in the format.
-   - \c [
-     Matches a nonempty sequence of characters from the specified set
-     of accepted characters; the next pointer must be a pointer to \c
-     char, and there must be enough room for all the characters in the
-     string, plus a terminating \c NUL character.  The usual skip of
-     leading white space is suppressed.  The string is to be made up
-     of characters in (or not in) a particular set; the set is defined
-     by the characters between the open bracket \c [ character and a
-     close bracket \c ] character.  The set excludes those characters
-     if the first character after the open bracket is a circumflex
-     \c ^.  To include a close bracket in the set, make it the first
-     character after the open bracket or the circumflex; any other
-     position will end the set.  The hyphen character \c - is also
-     special; when placed between two other characters, it adds all
-     intervening characters to the set.  To include a hyphen, make it
-     the last character before the final close bracket.  For instance,
-     <tt>[^]0-9-]</tt> means the set of <em>everything except close
-     bracket, zero through nine, and hyphen</em>.  The string ends
-     with the appearance of a character not in the (or, with a
-     circumflex, in) set or when the field width runs out.
-   - \c p
-     Matches a pointer value (as printed by <tt>%p</tt> in printf()); the
-     next pointer must be a pointer to \c void.
-   - \c n
-     Nothing is expected; instead, the number of characters consumed
-     thus far from the input is stored through the next pointer, which
-     must be a pointer to \c int.  This is not a conversion, although it
-     can be suppressed with the \c * flag.
-
-     These functions return the number of input items assigned, which
-     can be fewer than provided for, or even zero, in the event of a
-     matching failure.  Zero indicates that, while there was input
-     available, no conversions were assigned; typically this is due
-     to an invalid input character, such as an alphabetic character
-     for a <tt>%d</tt> conversion.  The value \c EOF is returned if an input
-     failure occurs before any conversion such as an end-of-file
-     occurs.  If an error or end-of-file occurs after conversion has
-     begun, the number of conversions which were successfully
-     completed is returned.
-
-     By default, all the conversions described above are available
-     except the floating-point conversions, and the <tt>\%[</tt> conversion.
-     These conversions will be available in the extended version
-     provided by the library \c libscanf_flt.a.  Note that either of
-     these conversions requires the availability of a buffer that
-     needs to be obtained at run-time using malloc().  If this buffer
-     cannot be obtained, the operation is aborted, returning the
-     value \c EOF.  To link a program against the extended version,
-     use the following compiler flags in the link stage:
-
-     \code
-     -Wl,-u,vfscanf -lscanf_flt -lm
-     \endcode
-
-     A third version is available for environments that are tight
-     on space.  This version is provided in the library
-     \c libscanf_min.a, and can be requested using the following
-     options in the link stage:
-
-     \code
-     -Wl,-u,vfscanf -lscanf_min -lm
-     \endcode
-
-     In addition to the restrictions of the standard version, this
-     version implements no field width specification, no conversion
-     assignment suppression flag (\c *), no <tt>%n</tt> specification, and
-     no general format character matching at all.  All characters in
-     \c fmt that do not comprise a conversion specification will
-     simply be ignored, including white space (that is normally used
-     to consume \e any amount of white space in the input stream).
-     However, the usual skip of initial white space in the formats
-     that support it is implemented.
-*/
 extern int	vfscanf(FILE *__stream, const char *__fmt, va_list __ap);
 
 /**
