@@ -27,14 +27,15 @@
   POSSIBILITY OF SUCH DAMAGE.
 */
 
-/* $Id: feof.c,v 1.3 2005/09/06 18:49:15 joerg_wunsch Exp $ */
+/* $Id: feof.c,v 1.3.4.1 2009/04/01 23:11:57 arcanum Exp $ */
 
 #include <stdio.h>
-
+#include "sectionname.h"
 #include "stdio_private.h"
 
 #undef feof
 
+ATTRIBUTE_CLIB_SECTION
 int
 feof(FILE *stream)
 {
