@@ -27,12 +27,13 @@
   POSSIBILITY OF SUCH DAMAGE.
 */
 
-/* $Id: puts.c,v 1.3 2005/09/06 18:49:15 joerg_wunsch Exp $ */
+/* $Id: puts.c,v 1.3.4.1 2009/04/01 23:12:00 arcanum Exp $ */
 
 #include <stdio.h>
-
+#include "sectionname.h"
 #include "stdio_private.h"
 
+ATTRIBUTE_CLIB_SECTION
 int
 puts(const char *str)
 {
