@@ -27,14 +27,15 @@
   POSSIBILITY OF SUCH DAMAGE.
 */
 
-/* $Id: sscanf.c,v 1.2 2005/08/17 14:00:56 joerg_wunsch Exp $ */
+/* $Id: sscanf.c,v 1.2.4.1 2009/04/01 23:12:05 arcanum Exp $ */
 
 #include <limits.h>
 #include <stdarg.h>
 #include <stdio.h>
-
+#include "sectionname.h"
 #include "stdio_private.h"
 
+ATTRIBUTE_CLIB_SECTION
 int
 sscanf(const char *s, const char *fmt, ...)
 {
