@@ -27,12 +27,14 @@
   POSSIBILITY OF SUCH DAMAGE.
 */
 
-/* $Id: vsnprintf.c,v 1.2 2007/03/17 23:11:28 dmix Exp $ */
+/* $Id: vsnprintf.c,v 1.2.2.1 2009/04/01 23:12:07 arcanum Exp $ */
 
 #include <limits.h>
 #include <stdarg.h>
 #include <stdio.h>
+#include "sectionname.h"
 
+ATTRIBUTE_CLIB_SECTION
 int
 vsnprintf(char *s, size_t n, const char *fmt, va_list ap)
 {
